@@ -19,9 +19,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -227,20 +225,9 @@ wnck_pager_init (WnckPager *pager)
     pager->priv->screen_connections[i] = 0;
 
   pager->priv->prelight = -1;
-  pager->priv->prelight_dnd = FALSE;
-
-  pager->priv->dragging = FALSE;
-  pager->priv->drag_start_x = 0;
-  pager->priv->drag_start_y = 0;
-  pager->priv->drag_window = NULL;
-
-  pager->priv->bg_cache = NULL;
 
   pager->priv->layout_manager_token = WNCK_NO_MANAGER_TOKEN;
 
-  pager->priv->dnd_activate = 0;
-  pager->priv->dnd_time = 0;
-
   g_object_set (pager, "has-tooltip", TRUE, NULL);
 
   gtk_drag_dest_set (GTK_WIDGET (pager), 0, targets, G_N_ELEMENTS (targets), GDK_ACTION_MOVE);
@@ -998,7 +985,7 @@ draw_window (cairo_t            *cr,
   GdkPixbuf *icon;
   int icon_x, icon_y, icon_w, icon_h;
   gboolean is_active;
-  GdkRGBA bg, fg;
+  GdkRGBA fg;
   gdouble translucency;
 
   context = gtk_widget_get_style_context (widget);
@@ -1006,27 +993,25 @@ draw_window (cairo_t            *cr,
   is_active = wnck_window_is_active (win);
   translucency = translucent ? 0.4 : 1.0;
 
-  cairo_save (cr);
-  cairo_rectangle (cr, winrect->x, winrect->y, winrect->width, winrect->height);
-  cairo_clip (cr);
+  gtk_style_context_save (context);
+  gtk_style_context_set_state (context, state);
 
-  gtk_style_context_get_background_color (context, state, &bg);
-  if (is_active) {
-    GtkSymbolicColor *c1, *c2;
+  cairo_push_group (cr);
 
-    c1 = gtk_symbolic_color_new_literal (&bg);
-    c2 = gtk_symbolic_color_new_shade (c1, 1.3);
-    gtk_symbolic_color_resolve (c2, NULL, &bg);
-    gtk_symbolic_color_unref (c2);
-    gtk_symbolic_color_unref (c1);
-  }
+  gtk_render_background (context, cr, winrect->x + 1, winrect->y + 1,
+                         MAX (0, winrect->width - 2), MAX (0, winrect->height - 2));
 
-  bg.alpha = translucency;
-  gdk_cairo_set_source_rgba (cr, &bg);
-  cairo_rectangle (cr,
-                   winrect->x + 1, winrect->y + 1,
-                   MAX (0, winrect->width - 2), MAX (0, winrect->height - 2));
-  cairo_fill (cr);
+  if (is_active)
+    {
+      /* Sharpen the foreground color */
+      cairo_set_source_rgba (cr, 1.0f, 1.0f, 1.0f, 0.3f);
+      cairo_rectangle (cr, winrect->x + 1, winrect->y + 1,
+                       MAX (0, winrect->width - 2), MAX (0, winrect->height - 2));
+      cairo_fill (cr);
+    }
+
+  cairo_pop_group_to_source (cr);
+  cairo_paint_with_alpha (cr, translucency);
 
   icon = wnck_window_get_icon (win);
 
@@ -1063,14 +1048,18 @@ draw_window (cairo_t            *cr,
       icon_x = winrect->x + (winrect->width - icon_w) / 2;
       icon_y = winrect->y + (winrect->height - icon_h) / 2;
 
-      cairo_save (cr);
-      gdk_cairo_set_source_pixbuf (cr, icon, icon_x, icon_y);
-      cairo_rectangle (cr, icon_x, icon_y, icon_w, icon_h);
-      cairo_clip (cr);
+      cairo_push_group (cr);
+      gtk_render_icon (context, cr, icon, icon_x, icon_y);
+      cairo_pop_group_to_source (cr);
       cairo_paint_with_alpha (cr, translucency);
-      cairo_restore (cr);
     }
 
+  cairo_push_group (cr);
+  gtk_render_frame (context, cr, winrect->x + 0.5, winrect->y + 0.5,
+                    MAX (0, winrect->width - 1), MAX (0, winrect->height - 1));
+  cairo_pop_group_to_source (cr);
+  cairo_paint_with_alpha (cr, translucency);
+
   gtk_style_context_get_color (context, state, &fg);
   fg.alpha = translucency;
   gdk_cairo_set_source_rgba (cr, &fg);
@@ -1080,7 +1069,7 @@ draw_window (cairo_t            *cr,
                    MAX (0, winrect->width - 1), MAX (0, winrect->height - 1));
   cairo_stroke (cr);
 
-  cairo_restore (cr);
+  gtk_style_context_restore (context);
 }
 
 static WnckWindow *
@@ -1220,28 +1209,33 @@ workspace_at_point (WnckPager *pager,
 }
 
 static void
-get_dark_color_for_state (GtkStyleContext *context,
-			  GtkStateFlags    state,
-			  GdkRGBA         *dark)
+draw_dark_rectangle (GtkStyleContext *context,
+                     cairo_t *cr,
+                     GtkStateFlags state,
+                     int rx, int ry, int rw, int rh)
 {
-  GdkRGBA           bg;
-  GtkSymbolicColor *c1, *c2;
+  gtk_style_context_save (context);
 
-  gtk_style_context_get_background_color (context, state, &bg);
+  gtk_style_context_set_state (context, state);
 
-  c1 = gtk_symbolic_color_new_literal (&bg);
+  cairo_push_group (cr);
 
-  c2 = gtk_symbolic_color_new_shade (c1, 0.7);
-  gtk_symbolic_color_resolve (c2, NULL, dark);
-  gtk_symbolic_color_unref (c2);
-  gtk_symbolic_color_unref (c1);
+  gtk_render_background (context, cr, rx, ry, rw, rh);
+  cairo_set_source_rgba (cr, 0.0f, 0.0f, 0.0f, 0.3f);
+  cairo_rectangle (cr, rx, ry, rw, rh);
+  cairo_fill (cr);
+
+  cairo_pop_group_to_source (cr);
+  cairo_paint (cr);
+
+  gtk_style_context_restore (context);
 }
 
 static void
 wnck_pager_draw_workspace (WnckPager    *pager,
                            cairo_t      *cr,
-			   int           workspace,
-			   GdkRectangle *rect,
+                           int           workspace,
+                           GdkRectangle *rect,
                            GdkPixbuf    *bg_pixbuf)
 {
   GList *windows;
@@ -1251,7 +1245,6 @@ wnck_pager_draw_workspace (WnckPager    *pager,
   GtkWidget *widget;
   GtkStateFlags state;
   GtkStyleContext *context;
-  GdkRGBA color;
 
   space = wnck_screen_get_workspace (pager->priv->screen, workspace);
   if (!space)
@@ -1280,10 +1273,8 @@ wnck_pager_draw_workspace (WnckPager    *pager,
     {
       if (!wnck_workspace_is_virtual (space))
         {
-	  get_dark_color_for_state (context, state, &color);
-          gdk_cairo_set_source_rgba (cr, &color);
-          cairo_rectangle (cr, rect->x, rect->y, rect->width, rect->height);
-          cairo_fill (cr);
+          draw_dark_rectangle (context, cr, state,
+                               rect->x, rect->y, rect->width, rect->height);
         }
       else
         {
@@ -1344,18 +1335,15 @@ wnck_pager_draw_workspace (WnckPager    *pager,
                     {
                       /* "+ j" is for the thin lines */
                       vy = rect->y + (height_ratio * screen_height) * j + j;
+                      GtkStateFlags rec_state = GTK_STATE_FLAG_NORMAL;
 
                       if (j == verti_views - 1)
                         vh = rect->height + rect->y - vy;
 
                       if (active_i == i && active_j == j)
-			get_dark_color_for_state (context, GTK_STATE_FLAG_SELECTED, &color);
-		      else
-			get_dark_color_for_state (context, GTK_STATE_FLAG_NORMAL, &color);
+                        rec_state = GTK_STATE_FLAG_SELECTED;
 
-		      gdk_cairo_set_source_rgba (cr, &color);
-                      cairo_rectangle (cr, vx, vy, vw, vh);
-                      cairo_fill (cr);
+                      draw_dark_rectangle (context, cr, rec_state, vx, vy, vw, vh);
                     }
                 }
             }
@@ -1365,10 +1353,8 @@ wnck_pager_draw_workspace (WnckPager    *pager,
               height_ratio = rect->height / (double) workspace_height;
 
               /* first draw non-active part of the viewport */
-	      get_dark_color_for_state (context, GTK_STATE_FLAG_NORMAL, &color);
-              gdk_cairo_set_source_rgba (cr, &color);
-              cairo_rectangle (cr, rect->x, rect->y, rect->width, rect->height);
-              cairo_fill (cr);
+              draw_dark_rectangle (context, cr, GTK_STATE_FLAG_NORMAL,
+                                   rect->x, rect->y, rect->width, rect->height);
 
               if (is_current)
                 {
@@ -1380,10 +1366,8 @@ wnck_pager_draw_workspace (WnckPager    *pager,
                   vw = width_ratio * screen_width;
                   vh = height_ratio * screen_height;
 
-		  get_dark_color_for_state (context, GTK_STATE_FLAG_SELECTED, &color);
-                  gdk_cairo_set_source_rgba (cr, &color);
-                  cairo_rectangle (cr, vx, vy, vw, vh);
-                  cairo_fill (cr);
+                  draw_dark_rectangle (context, cr, GTK_STATE_FLAG_SELECTED,
+                                       vx, vy, vw, vh);
                 }
             }
         }
@@ -1395,51 +1379,44 @@ wnck_pager_draw_workspace (WnckPager    *pager,
 							    wnck_screen_get_workspace (pager->priv->screen,
 										       workspace));
 
-      tmp = windows;
-      while (tmp != NULL)
-	{
-	  WnckWindow *win = tmp->data;
-	  GdkRectangle winrect;
+  tmp = windows;
+  while (tmp != NULL)
+  	{
+  	  WnckWindow *win = tmp->data;
+  	  GdkRectangle winrect;
 
-	  get_window_rect (win, rect, &winrect);
+  	  get_window_rect (win, rect, &winrect);
 
-	  draw_window (cr,
-		       widget,
-		       win,
-		       &winrect,
-                       state,
-		       win == pager->priv->drag_window && pager->priv->dragging ? TRUE : FALSE);
+      gboolean translucent = (win == pager->priv->drag_window) && pager->priv->dragging;
+      draw_window (cr, widget, win, &winrect, state, translucent);
 
-	  tmp = tmp->next;
-	}
+  	  tmp = tmp->next;
+  	}
 
       g_list_free (windows);
     }
   else
     {
       /* Workspace name mode */
+      GtkStateFlags layout_state;
       const char *workspace_name;
       PangoLayout *layout;
+      WnckWorkspace *ws;
       int w, h;
 
-      workspace_name = wnck_workspace_get_name (wnck_screen_get_workspace (pager->priv->screen,
-									   workspace));
-      layout = gtk_widget_create_pango_layout  (widget,
-						workspace_name);
+      ws = wnck_screen_get_workspace (pager->priv->screen, workspace);
+      workspace_name = wnck_workspace_get_name (ws);
+      layout = gtk_widget_create_pango_layout (widget, workspace_name);
 
       pango_layout_get_pixel_size (layout, &w, &h);
 
-      if (is_current)
-	gtk_style_context_get_color (context, GTK_STATE_FLAG_SELECTED, &color);
-      else
-	gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &color);
-      gdk_cairo_set_source_rgba (cr, &color);
-      cairo_move_to (cr,
-                     rect->x + (rect->width - w) / 2,
-                     rect->y + (rect->height - h) / 2);
-
-      pango_cairo_show_layout (cr, layout);
+      layout_state = (is_current) ? GTK_STATE_FLAG_SELECTED : GTK_STATE_FLAG_NORMAL;
+      gtk_style_context_save (context);
+      gtk_style_context_set_state (context, layout_state);
+      gtk_render_layout (context, cr, rect->x + (rect->width - w) / 2,
+                         rect->y + (rect->height - h) / 2, layout);
 
+      gtk_style_context_restore (context);
       g_object_unref (layout);
     }
 
@@ -1447,18 +1424,13 @@ wnck_pager_draw_workspace (WnckPager    *pager,
     {
       gtk_style_context_save (context);
       gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
-
-      cairo_save (cr);
       gtk_render_frame (context, cr, rect->x, rect->y, rect->width, rect->height);
-      cairo_restore (cr);
-
       gtk_style_context_restore (context);
 
       cairo_set_source_rgb (cr, 0.0, 0.0, 0.0); /* black */
       cairo_set_line_width (cr, 1.0);
-      cairo_rectangle (cr,
-		       rect->x + 0.5, rect->y + 0.5,
-		       MAX (0, rect->width - 1), MAX (0, rect->height - 1));
+      cairo_rectangle (cr, rect->x + 0.5, rect->y + 0.5,
+                           MAX (0, rect->width - 1), MAX (0, rect->height - 1));
       cairo_stroke (cr);
     }
 }
@@ -1679,10 +1651,9 @@ wnck_pager_drag_motion (GtkWidget          *widget,
   if (gtk_drag_dest_find_target (widget, context, NULL))
     {
 #if GTK_CHECK_VERSION(2,21,0)
-       gdk_drag_status (context,
-                        gdk_drag_context_get_suggested_action (context), time);
+      gdk_drag_status (context, gdk_drag_context_get_suggested_action (context), time);
 #else
-       gdk_drag_status (context, context->suggested_action, time);
+      gdk_drag_status (context, context->suggested_action, time);
 #endif
     }
   else
@@ -1690,21 +1661,21 @@ wnck_pager_drag_motion (GtkWidget          *widget,
       gdk_drag_status (context, 0, time);
 
       if (pager->priv->prelight != previous_workspace &&
-	  pager->priv->dnd_activate != 0)
-	{
-	  /* remove timeout, the window we hover over changed */
-	  g_source_remove (pager->priv->dnd_activate);
-	  pager->priv->dnd_activate = 0;
-	  pager->priv->dnd_time = 0;
-	}
+          pager->priv->dnd_activate != 0)
+        {
+          /* remove timeout, the window we hover over changed */
+          g_source_remove (pager->priv->dnd_activate);
+          pager->priv->dnd_activate = 0;
+          pager->priv->dnd_time = 0;
+        }
 
       if (pager->priv->dnd_activate == 0 && pager->priv->prelight > -1)
-	{
-	  pager->priv->dnd_activate = g_timeout_add (WNCK_ACTIVATE_TIMEOUT,
-                                                     wnck_pager_drag_motion_timeout,
-                                                     pager);
-	  pager->priv->dnd_time = time;
-	}
+        {
+          pager->priv->dnd_activate = g_timeout_add_seconds (WNCK_ACTIVATE_TIMEOUT,
+                                                             wnck_pager_drag_motion_timeout,
+                                                             pager);
+          pager->priv->dnd_time = time;
+        }
     }
 
   return (pager->priv->prelight != -1);
@@ -1880,8 +1851,7 @@ wnck_update_drag_icon (WnckWindow     *window,
                                                CAIRO_CONTENT_COLOR,
                                                rect.width, rect.height);
   cr = cairo_create (surface);
-  draw_window (cr, widget, window,
-	       &rect, GTK_STATE_FLAG_NORMAL, FALSE);
+  draw_window (cr, widget, window, &rect, GTK_STATE_FLAG_NORMAL, FALSE);
   cairo_destroy (cr);
   cairo_surface_set_device_offset (surface, 2, 2);
 
@@ -1975,12 +1945,16 @@ wnck_pager_motion (GtkWidget        *widget,
 {
   WnckPager *pager;
   GdkWindow *window;
+  GdkDeviceManager *dev_manager;
+  GdkDevice *pointer;
   int x, y;
 
   pager = WNCK_PAGER (widget);
 
+  dev_manager = gdk_display_get_device_manager (gtk_widget_get_display (widget));
   window = gtk_widget_get_window (widget);
-  gdk_window_get_pointer (window, &x, &y, NULL);
+  pointer = gdk_device_manager_get_client_pointer (dev_manager);
+  gdk_window_get_device_position (window, pointer, &x, &y, NULL);
 
   if (!pager->priv->dragging &&
       pager->priv->drag_window != NULL &&
@@ -2090,6 +2064,8 @@ wnck_pager_scroll_event (GtkWidget      *widget,
   int                 n_workspaces;
   int                 n_columns;
   int                 in_last_row;
+  gdouble             smooth_x;
+  gdouble             smooth_y;
 
   pager = WNCK_PAGER (widget);
 
@@ -2122,6 +2098,13 @@ wnck_pager_scroll_event (GtkWidget      *widget,
           case GDK_SCROLL_LEFT:
             absolute_direction = GDK_SCROLL_RIGHT;
             break;
+          case GDK_SCROLL_SMOOTH:
+            gdk_event_get_scroll_deltas ((GdkEvent*)event, &smooth_x, &smooth_y);
+            if (smooth_x > 5)
+              absolute_direction = GDK_SCROLL_RIGHT;
+            else if (smooth_x < -5)
+              absolute_direction = GDK_SCROLL_LEFT;
+            break;
         }
     }
 
