@@ -16,9 +16,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -1367,6 +1365,23 @@ _wnck_get_frame_extents (Screen *screen,
       retval = TRUE;
     }
 
+  if (p_size == NULL)
+    {
+      _wnck_get_cardinal_list (screen, xwindow,
+                               _wnck_atom_get ("_GTK_FRAME_EXTENTS"),
+                               &p_size, &n_size);
+
+      if (p_size != NULL && n_size == 4)
+        {
+          *left_frame   = -p_size[0];
+          *right_frame  = -p_size[1];
+          *top_frame    = -p_size[2];
+          *bottom_frame = -p_size[3];
+
+          retval = TRUE;
+        }
+    }
+
   if (p_size != NULL)
     g_free (p_size);
 
